!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!! Reading COSMO Topography Binary Data (0.005 deg. = 500 m data)   !!!
!!! Last Updated: March 15, 2022 (DBS/RS).                           !!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

	Parameter (nxc = 21, nyc = 21, nxp = 102, nyp = 102)
	Parameter (nxnyc = nxc*nyc, nxnyp = nxp*nyp)
	!!! nxc, nyc = COSMO Grid Points
	!!! nxp, nyp = PALM Grid Points

	Real, Dimension(nxc,nyc)	:: ztopo_cosmo, clat, clon
	Real, Dimension(nxp,nyp)	:: ztopo_palm, plat, plon
	Real, Dimension(nxnyc)		:: ztopoc
	Real, Dimension(nxnyp)		:: ztopop

	Open (11, file = 'cosmo-topo.bin', form='unformatted', access='stream')
	Open (12, file = 'cosmo-topo.dat', form='formatted')
	Open (13, file = 'palm-topo-100.dat', form='formatted')
	Open (14, file = 'COSMO-XY.dat', form='formatted')
	Open (15, file = 'PALM-XY-100.dat', form='formatted')

	Open (21, file = 'COSMO-Topography.grb', form='unformatted', &
                  access='stream')
	Open (22, file = 'PALM-Topography-100.grb', form='unformatted',  &
                  access='stream')

	Write (14, 50) 'LON', 'LAT', 'ALT'
	Write (15, 50) 'LON', 'LAT', 'ALT'
 50	Format (3(1x, a12))

	Read (11) ztopoc

	dxc = 0.1/float(nxc-1)
	dyc = 0.1/float(nyc-1)

	dxp = 0.1/float(nxp-1)
	dyp = 0.1/float(nyp-1)

	Do jyc = 1, nyc
	Do ixc = 1, nxc
	kc = ixc + ((jyc-1)*nyc)
	clon(ixc,jyc) = 76.95 + ((ixc-1)*dxc) 
	clat(ixc,jyc) =  8.45 + ((jyc-1)*dyc) 
	ztopo_cosmo(ixc,jyc) = ztopoc(kc)
	Enddo
	Enddo

	Do ixp = 1, nxp
	Do jyp = 1, nyp
	plon(ixp,jyp) = 76.95 + ((ixp-1)*dxp)
	plat(ixp,jyp) =  8.45 + ((jyp-1)*dyp)
	Enddo
	Enddo

	Call Bilinear_Ipolation(clon,clat,ztopo_cosmo,nxc,nyc,plon,plat,ztopo_palm,nxp,nyp)

	Do ixp = 1, nxp
	Do jyp = 1, nyp
	kp = ixp + ((jyp-1)*nyp)
	ztopop(kp) = ztopo_palm(ixp,jyp)
	Enddo
	Enddo

	Do jyc = nyc, 1, -1
	Write (12, 10) (ztopo_cosmo(ixc,jyc),ixc=1,nxc)
 10	Format (21(1x, f6.1))
	Enddo

	Do jyp = nyp, 1, -1
	Write (13, 11) (ztopo_palm(ixp,jyp),ixp=1,nxp)
 11	Format (102(1x, f6.1))
	Enddo

	Do iyc = 1, nyc
	Do ixc = 1, nxc
	Write (14, 12) clon(ixc,iyc), clat(ixc,iyc), ztopo_cosmo(ixc,iyc)
 12	Format (2(1x, f12.5), 1x, f12.4)
	Enddo
	Write (14, *)
	Enddo

	Do iyp = 1, nyp
	Do ixp = 1, nxp
	Write (15, 13) plon(ixp,iyp), plat(ixp,iyp), ztopo_palm(ixp,iyp)
 13	Format (2(1x, f12.5), 1x, f12.4)
	Enddo
	Write (15, *)
	Enddo

	Write (21) ztopoc
	Write (22) ztopop

	Print *, ztopo_palm(51,51)

	Stop

	End

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!! Subroutine for "BI-LINEAR INTERPOLATION"                           !
!!! This Subroutine is useful for fine-gridding a coarse-grid 2-D data !
!!! Last Updated: DBS (December 01, 2019)                              !
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!! Inputs to the Subroutine:					       !
!!! 	a1(m1,n1)	: X-variable				       !
!!! 	b1(m1,n1)	: Y-variable				       !
!!! 	c1(m1,n1)	: Z-variable				       !
!!! 	m1		: No. of grid-points along X-axis	       !
!!! 	n1		: No. of grid-points along Y-axis	       !
!!! 	a2(m2,n2)	: X-variable				       !
!!! 	b2(m2,n2)	: Y-variable				       !
!!! 	m2		: No. of new grid-points along X-axis 	       !
!!! 	n2		: No. of new grid-points along Y-axis          !
!!! Outputs of the Subroutine:					       !
!!! 	c2(m2,n2)	: Z-variable				       !
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

	Subroutine Bilinear_Ipolation (a1,b1,c1,m1,n1,a2,b2,c2,m2,n2)

	Real, Dimension(m1,n1)	:: a1, b1, c1
	Real, Dimension(m2,n2)	:: a2, b2, c2

	Real, Dimension(m1-1,n1-1)	:: dcda1, dcdb1
	Real, Dimension(m2-1,n2-1)	:: dcda2, dcdb2

	da1 = (a1(m1,1) - a1(1,1))/(m1-1) !!! Pre-Assumed: Uniform X-grids
	db1 = (b1(1,n1) - b1(1,1))/(n1-1) !!! Pre-Assumed: Uniform Y-grids

	da2 = (a2(m2,1) - a2(1,1))/(m2-1)
	db2 = (b2(1,n2) - b2(1,1))/(n2-1)

 51	Format (4(1x, f12.6))

	Do j1 = 1, n1-1
	Do i1 = 1, m1-1
	deltax = c1(i1+1,j1) - c1(i1,j1)
	deltay = c1(i1,j1+1) - c1(i1,j1)
	dcda1(i1,j1)  = deltax/da1
	dcdb1(i1,j1)  = deltay/db1
	Enddo
	Enddo

	Do i2 = 1, m2
	Do j2 = 1, n2
	i1 = (((i2-1)*da2)/da1)+1
	j1 = (((j2-1)*db2)/db1)+1
!	deltax1 = a2(i2,j2) - a1(i1,j1)
!	deltay1 = b2(i2,j2) - b1(i1,j1)
!	c2(i2,j2) = c1(i1,j1) + (deltax1*dcda1(i1,j1)) + (deltay1*dcdb1(i1,j1))
!	If (c2(i2,j2).lt.10.0) c2(i2,j2) = 10.0
!	If (c2(i2,j2).gt.110.0) c2(i2,j2) = 110.0
	c2(i2,j2) = c1(i1,j1)
	Enddo
	Enddo

	Return

	End
